// Path block info
//
// Handles all the path blocking code
// Initialization is not instant
// This object is always initialized within the level
//
// by Higor
class sgPathBlock expands Object;

var float ScanDist;

var native NavigationPoint N;
var int iPoll;
var int BlockedReachSpecs[32];
var int iBlock;
var bool bInitialized;

function Init()
{
	if ( (sgBuilding(Outer) == none) || (iBlock == ArrayCount(BlockedReachSpecs)) )
		return;
	N = sgBuilding(Outer).Level.NavigationPointList;
	iPoll = 0;
	bInitialized = false;
}

function PollBlock()
{
	local int PollCounter;
	local Actor Start, End;
	local int ReachFlags, Distance, Count;
	local vector Q;

	assert( sgBuilding(Outer) != none );

	POLL_NAVIG:
	if ( PollCounter++ >= 20 ) //20 or more loops maximum to ensure stable framerate
		return;
	if ( N != none )
	{
		if ( VSize(N.Location - sgBuilding(Outer).Location) <= ScanDist )
		{
//			Log("===== Polling "$N.Name); //DELETEME
			if ( iPoll == 0 && PathInBuilding() ) //Block all incoming paths
			{
//				Log("Path in building"); //DELETEME
				PollCounter += 3;
				//Remove all upstream paths
				For ( iPoll=0 ; iPoll<16 ; iPoll++ )
				{
					if ( N.upstreamPaths[iPoll] >= 0 )
					{
//						Log("Getting rid of reachspec "$N.upstreamPaths[iPoll]@"from"@iPoll); //DELETEME
						N.describeSpec( N.upstreamPaths[iPoll], Start, End, ReachFlags, Distance);
						BlockedReachSpecs[iBlock++] = N.upstreamPaths[iPoll];
						//Remove path from Start
						if ( NavigationPoint(Start) != none )
							RemovePath( NavigationPoint(Start), N.upstreamPaths[iPoll] );
						N.upstreamPaths[iPoll] = -1;
						if ( iBlock == ArrayCount(BlockedReachSpecs) )
							goto FORCE_INIT;
					}
				}
				iPoll = 0;
			}
			else if ( (iPoll < 16) && (N.Paths[iPoll] >= 0) )
			{
//				Log("Checking spec"@N.Paths[iPoll]); //DELETEME
				N.describeSpec( N.Paths[iPoll], Start, End, ReachFlags, Distance);
				PollCounter += 4;
				if ( (End != none) && (VSize(End.Location - sgBuilding(Outer).Location) < ScanDist) && BlockingLine( N.Location, End.Location) )
				{
//					Log("Getting rid of spec in slot"@iPoll); //DELETEME
					BlockedReachSpecs[iBlock++] = N.Paths[iPoll];
					if ( NavigationPoint(End) != none )
						RemoveUsPath( NavigationPoint(End), N.Paths[iPoll] );
					RemovePath( N, N.Paths[iPoll], iPoll);
					if ( iBlock == ArrayCount(BlockedReachSpecs) )
						goto FORCE_INIT;
					PollCounter++;
				}
				iPoll++;
				Goto POLL_NAVIG;
			}
			else
				iPoll = 0;
		}
		N = N.nextNavigationPoint;
		Goto POLL_NAVIG;
	}
	if ( false )
	{
		FORCE_INIT:
//		Log("Maximum limit reached!!!"); //DELETEME
	}
	bInitialized = true;
}

function DeInit()
{
	local Actor Start, End;
	local int ReachFlags, Distance, i, Count;
	
	assert( sgBuilding(Outer) != none );
	if ( sgBuilding(Outer).Level.NavigationPointList == none )
	{
		Log("Something broke the Navigation point list!",'SiegeIV');
		return;
	}
	
	For ( i=0 ; i<iBlock ; i++ )
	{
		sgBuilding(Outer).Level.NavigationPointList.describeSpec( BlockedReachSpecs[i], Start, End, ReachFlags, Distance);
		// For deletable path support (FerBotz)
		if ( (NavigationPoint(Start) != none) && !Start.bDeleteMe )
		{
			if ( !AddPath( NavigationPoint(Start), BlockedReachSpecs[i]) )
			{}
		}
		if ( (NavigationPoint(End) != none) && !End.bDeleteMe )
		{
			if ( !AddUsPath( NavigationPoint(End), BlockedReachSpecs[i]) )
			{}
		}
	}
}


// MOVE THESE TO SIEGESTATICS!
//False means there was an alteration, let a general handler fix this later
static function bool AddPath( NavigationPoint NE, int Spec)
{
	local int i;
	while ( i<16 && NE.Paths[i] >= 0 )
		i++;
	if ( i < 16 )
	{
		NE.Paths[i] = Spec;
		return true;
	}
}
static function bool AddUsPath( NavigationPoint NE, int Spec)
{
	local int i;
	while ( i<16 && NE.upstreamPaths[i] >= 0 )
		i++;
	if ( i < 16 )
	{
		NE.upstreamPaths[i] = Spec;
		return true;
	}
}
static function RemovePath( NavigationPoint NE, int Spec, optional int i)
{
	while ( i<16 && NE.Paths[i] != Spec ) //Find slot
		i++;
	if ( i == 16 )
		return;
	NE.Paths[i++] = -1; //Null slot
	if ( NE.Paths[i] >= 0 ) //There's more stuff
	{
		while ( i<15 && NE.Paths[i] >= 0 ) //Move valid slots one below
			NE.Paths[i-1] = NE.Paths[i++];
		NE.Paths[i-1] = -1; //Null last valid
	}
}
static function RemoveUsPath( NavigationPoint NE, int Spec)
{
	local int i;
	i=0;
	while ( i<16 && NE.upstreamPaths[i] != Spec ) //Find slot
		i++;
	if ( i == 16 )
		return;
	NE.upstreamPaths[i++] = -1; //Null slot
	if ( NE.upstreamPaths[i] >= 0 ) //There's more stuff
	{
		while ( i<15 && NE.upstreamPaths[i] >= 0 ) //Move valid slots one below
			NE.upstreamPaths[i-1] = NE.upstreamPaths[i++];
		NE.upstreamPaths[i-1] = -1; //Null last valid
	}
}

function bool PathInBuilding()
{
	local vector aVec;
	aVec = sgBuilding(Outer).Location - N.Location;
	if ( VSize(aVec * vect(1,1,0)) > sgBuilding(Outer).CollisionRadius + 10 )
		return false;
	if ( aVec.Z > sgBuilding(Outer).CollisionHeight + 40 ) //Not above path, it is reachable
		return false;
	return -aVec.Z < sgBuilding(Outer).CollisionHeight + 10; //Barely above path, count it as reachable
}

function bool BlockingLine( vector A, vector B)
{
	local Actor Act;
	local vector HitLocation, HitNormal;
	
	//Extent trace
	ForEach sgBuilding(Outer).Level.TraceActors (class'Actor', Act, HitLocation, HitNormal, B, A )
		if ( Act == sgBuilding(Outer) ) //Building was hit
			return true;
}

/*
function bool BlockingLine( vector A, vector B, float MaxDist)
{
	local vector Q;

	if ( FMax(VSize(A - Location), VSize(B-Location)) > MaxDist )
		return false;
	Q = A + Normal( B - A ) * ((( B - A ) dot ( Location - A )) / VSize( A - B ));
	if ( HSize( Q-Location) > CollisionRadius )
		return false;
	return (abs(Location.Z - Q.Z) < CollisionHeight);
}
*/

defaultproperties
{
    BlockedReachSpecs(0)=-1
    BlockedReachSpecs(1)=-1
    BlockedReachSpecs(2)=-1
    BlockedReachSpecs(3)=-1
    BlockedReachSpecs(4)=-1
    BlockedReachSpecs(5)=-1
    BlockedReachSpecs(6)=-1
    BlockedReachSpecs(7)=-1
    BlockedReachSpecs(8)=-1
    BlockedReachSpecs(9)=-1
    BlockedReachSpecs(10)=-1
    BlockedReachSpecs(11)=-1
    BlockedReachSpecs(12)=-1
    BlockedReachSpecs(13)=-1
    BlockedReachSpecs(14)=-1
    BlockedReachSpecs(15)=-1
    ScanDist=800.0
}